#include <curl/curl.h>
#include <iostream>
#include <cmath>
#include <vector>
#include <sstream>
#include <string>
#include "histogram.h"
#include "text.h"
#include "svg.h"

using namespace std;


struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

 int progress_callback(void *clientp, double dltotal, double dlnow, double ultotal, double ulnow)
{
    if(dltotal > 0) {
        double progress = ((dlnow * 100) / dltotal);
        cerr << "Progress: " << progress << "%" << endl;
    }
    return 0;
}


size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
     size_t data_size = item_size * item_count;


    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);

    return data_size;
}


Input
input_data(istream& in, bool prompt)
{
    if(prompt)
        cerr << "count=";
    size_t number_count;
    in >> number_count;

    Input rez;

    if(prompt)
        cerr << "numbers= ";
    rez.numbers.resize(number_count);

    for (size_t i = 0; i < number_count; i++)
    {
       in >> rez.numbers[i];
    }
    if(prompt)
        cerr << "bin_count= ";
    in >> rez.bin_count;
    return rez;
}

Input
download(const string& address) {
    stringstream buffer;

    CURL* curl = curl_easy_init();
    if(curl)
    {
        CURLcode res;
         curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
        curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, progress_callback);

        res = curl_easy_perform(curl);
        if (res != 0)
            {
            cout << curl_easy_strerror(res);
            exit(1);
            }

        curl_easy_cleanup(curl);
    }
    return input_data(buffer, false);
}

int main(int argc, char* argv[]){

Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(cin, true);
    }


    auto bins = make_histogram(input.numbers, input.bin_count);
    double ttt=0.0;
     show_histogram_svg(bins,ttt);
   /* show_histogram_text(bins, in.bin_count);*/

    return 0;
}
